@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -50,8 +50,7 @@
  * automatically expanded to include as many extra digits as necessary
  * to avoid losing information.
  */
-#ifndef INCLUDED_AMOUNT_H
-#define INCLUDED_AMOUNT_H
+#pragma once
 
 #include "utils.h"
 #include "times.h"
@@ -77,7 +76,7 @@ enum parse_flags_enum_t {
   PARSE_SOFT_FAIL  = 0x80
 };
 
-typedef basic_flags_t<parse_flags_enum_t, uint_least8_t> parse_flags_t;
+typedef flags::basic_t<parse_flags_enum_t, uint_least8_t> parse_flags_t;
 
 /**
  * @brief Encapsulate infinite-precision commoditized amounts
@@ -348,7 +347,7 @@ public:
 
   amount_t roundto(int places) const {
     amount_t temp(*this);
-    temp.in_place_round();
+    temp.in_place_roundto(places);
     return temp;
   }
   void in_place_roundto(int places);
@@ -786,5 +785,3 @@ void put_amount(property_tree::ptree& pt, const amount_t& amt,
                 bool commodity_details = false);
 
 } // namespace ledger
-
-#endif // INCLUDED_AMOUNT_H
