@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2018, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -46,8 +46,7 @@
  * is designed to allow this, tracking the amounts of each component
  * commodity separately.
  */
-#ifndef _BALANCE_H
-#define _BALANCE_H
+#pragma once
 
 #include "amount.h"
 
@@ -60,7 +59,7 @@ DECLARE_EXCEPTION(balance_error, std::runtime_error);
  *
  * @brief A wrapper around amount_t allowing addition of multiple commodities.
  *
- * The balance_t class is appopriate for keeping a running balance
+ * The balance_t class is appropriate for keeping a running balance
  * where amounts of multiple commodities may be involved.
  */
 class balance_t
@@ -506,6 +505,9 @@ public:
   optional<amount_t>
   commodity_amount(const optional<const commodity_t&>& commodity = none) const;
 
+  amounts_map::iterator find_by_name(const commodity_t& comm);
+  amounts_map::const_iterator find_by_name(const commodity_t& comm) const;
+
   balance_t number() const {
     balance_t temp;
     foreach (const amounts_map::value_type& pair, amounts)
@@ -603,6 +605,6 @@ inline std::ostream& operator<<(std::ostream& out, const balance_t& bal) {
 
 void put_balance(property_tree::ptree& pt, const balance_t& bal);
 
-} // namespace ledger
+balance_t average_lot_prices(const balance_t& bal);
 
-#endif // _BALANCE_H
+} // namespace ledger
