@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -39,8 +39,7 @@
  *
  * @ingroup data
  */
-#ifndef INCLUDED_CONVERT_H
-#define INCLUDED_CONVERT_H
+#pragma once
 
 #include "value.h"
 
@@ -51,5 +50,3 @@ class call_scope_t;
 value_t convert_command(call_scope_t& scope);
 
 } // namespace ledger
-
-#endif // INCLUDED_CONVERT_H
