@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -237,8 +237,6 @@ void anonymize_posts::render_commodity(amount_t& amt)
 
 void anonymize_posts::operator()(post_t& post)
 {
-	boost::uuids::detail::sha1  sha;
-  unsigned int message_digest[5];
   bool         copy_xact_details = false;
 
   if (last_xact != post.xact) {
@@ -255,12 +253,7 @@ void anonymize_posts::operator()(post_t& post)
     std::ostringstream buf;
     buf << reinterpret_cast<boost::uintmax_t>(post.xact->payee.c_str())
         << integer_gen() << post.xact->payee.c_str();
-
-		sha.reset();
-    sha.process_bytes(buf.str().c_str(), buf.str().length());
-    sha.get_digest(message_digest);
-
-    xact.payee = to_hex(message_digest);
+    xact.payee = sha1sum(buf.str(), 8);
     xact.note  = none;
   } else {
     xact.journal = post.xact->journal;
@@ -273,12 +266,7 @@ void anonymize_posts::operator()(post_t& post)
        acct = acct->parent) {
     std::ostringstream buf;
     buf << integer_gen() << acct << acct->fullname();
-
-    sha.reset();
-    sha.process_bytes(buf.str().c_str(), buf.str().length());
-    sha.get_digest(message_digest);
-
-    account_names.push_front(to_hex(message_digest));
+    account_names.push_front(sha1sum(buf.str(), 8));
   }
 
   account_t * new_account =
@@ -974,7 +962,7 @@ void interval_posts::operator()(post_t& post)
   if (interval.duration) {
     all_posts.push_back(&post);
   }
-  else if (interval.find_period(post.date())) {
+  else if (interval.find_period(post.date(), align_intervals)) {
     item_handler<post_t>::operator()(post);
   }
 }
@@ -991,10 +979,10 @@ void interval_posts::flush()
                    sort_posts_by_date());
 
   // only if the interval has no start use the earliest post
-  if (!(interval.begin() && interval.find_period(*interval.begin())))
+  if (!(interval.begin() && interval.find_period(*interval.begin(), align_intervals)))
     // Determine the beginning interval by using the earliest post
     if (all_posts.size() > 0 && all_posts.front()
-        && !interval.find_period(all_posts.front()->date()))
+        && !interval.find_period(all_posts.front()->date(), align_intervals))
       throw_(std::logic_error, _("Failed to find period for interval report"));
 
   // Walk the interval forward reporting all posts within each one
@@ -1082,6 +1070,7 @@ namespace {
     }
 
     void operator()(const amount_t& amount) {
+      if(amount.is_zero()) return;
       post_t& balance_post = temps.create_post(xact, &balance_account);
       balance_post.amount = - amount;
       (*handler)(balance_post);
@@ -1268,7 +1257,7 @@ void budget_posts::report_budget_items(const date_t& date)
     foreach (pending_posts_list::iterator& i, posts_to_erase)
       pending_posts.erase(i);
   }
-  
+
   if (pending_posts.size() == 0)
     return;
 
@@ -1432,10 +1421,8 @@ void forecast_posts::flush()
         least = i;
     }
 
-#if !NO_ASSERTS
     if ((*least).first.finish)
       assert(*(*least).first.start < *(*least).first.finish);
-#endif
 
     // If the next date in the series for this periodic posting is more than 5
     // years beyond the last valid post we generated, drop it from further
