@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2018, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -39,8 +39,7 @@
  *
  * @ingroup report
  */
-#ifndef _FILTERS_H
-#define _FILTERS_H
+#pragma once
 
 #include "chain.h"
 #include "xact.h"
@@ -50,6 +49,8 @@
 
 namespace ledger {
 
+using namespace boost::placeholders;
+
 //////////////////////////////////////////////////////////////////////
 //
 // Posting collector
@@ -371,7 +372,7 @@ class anonymize_posts : public item_handler<post_t>
 public:
   anonymize_posts(post_handler_ptr handler)
     : item_handler<post_t>(handler), next_comm_id(0), last_xact(NULL),
-      rnd_gen(static_cast<unsigned int>(static_cast<uintmax_t>(std::time(0)))),
+      rnd_gen(static_cast<unsigned int>(static_cast<boost::uintmax_t>(std::time(0)))),
       integer_range(1, 2000000000L),
       integer_gen(rnd_gen, integer_range) {
     TRACE_CTOR(anonymize_posts, "post_handler_ptr");
@@ -426,6 +427,9 @@ public:
 
 class collapse_posts : public item_handler<post_t>
 {
+
+  typedef std::map<account_t *,value_t> totals_map;
+
   expr_t&             amount_expr;
   predicate_t         display_predicate;
   predicate_t         only_predicate;
@@ -434,8 +438,10 @@ class collapse_posts : public item_handler<post_t>
   xact_t *            last_xact;
   post_t *            last_post;
   temporaries_t       temps;
-  account_t *         totals_account;
+  account_t *         global_totals_account;
+  totals_map          totals;
   bool                only_collapse_if_zero;
+  unsigned short      collapse_depth;
   std::list<post_t *> component_posts;
   report_t&           report;
 
@@ -447,12 +453,14 @@ public:
                  expr_t&          _amount_expr,
                  predicate_t      _display_predicate,
                  predicate_t      _only_predicate,
-                 bool             _only_collapse_if_zero = false)
+                 bool             _only_collapse_if_zero = false,
+                 unsigned short   _collapse_depth = 0)
     : item_handler<post_t>(handler), amount_expr(_amount_expr),
       display_predicate(_display_predicate),
       only_predicate(_only_predicate), count(0),
       last_xact(NULL), last_post(NULL),
-      only_collapse_if_zero(_only_collapse_if_zero), report(_report) {
+      only_collapse_if_zero(_only_collapse_if_zero),
+      collapse_depth(_collapse_depth), report(_report) {
     create_accounts();
     TRACE_CTOR(collapse_posts, "post_handler_ptr, ...");
   }
@@ -462,9 +470,11 @@ public:
   }
 
   void create_accounts() {
-    totals_account = &temps.create_account(_("<Total>"));
+    global_totals_account = &temps.create_account(_("<Total>"));
   }
 
+  value_t& find_totals(account_t* account);
+
   virtual void flush() {
     report_subtotal();
     item_handler<post_t>::flush();
@@ -486,6 +496,7 @@ public:
 
     temps.clear();
     create_accounts();
+    totals.clear();
     component_posts.clear();
 
     item_handler<post_t>::clear();
@@ -720,6 +731,7 @@ class interval_posts : public subtotal_posts
   account_t *     empty_account;
   bool            exact_periods;
   bool            generate_empty_posts;
+  bool            align_intervals;
 
   std::deque<post_t *> all_posts;
 
@@ -731,10 +743,12 @@ public:
                  expr_t&                amount_expr,
                  const date_interval_t& _interval,
                  bool                   _exact_periods        = false,
-                 bool                   _generate_empty_posts = false)
+                 bool                   _generate_empty_posts = false,
+                 bool                   _align_intervals      = false)
     : subtotal_posts(_handler, amount_expr), start_interval(_interval),
       interval(start_interval), exact_periods(_exact_periods),
-      generate_empty_posts(_generate_empty_posts) {
+      generate_empty_posts(_generate_empty_posts),
+      align_intervals(_align_intervals) {
     create_accounts();
     TRACE_CTOR(interval_posts,
                "post_handler_ptr, expr_t&, date_interval_t, bool, bool");
@@ -780,13 +794,14 @@ class posts_as_equity : public subtotal_posts
   post_t *    last_post;
   account_t * equity_account;
   account_t * balance_account;
+  bool        unround;
 
   posts_as_equity();
 
 public:
   posts_as_equity(post_handler_ptr _handler, report_t& _report,
-                  expr_t& amount_expr)
-    : subtotal_posts(_handler, amount_expr), report(_report) {
+                  expr_t& amount_expr, bool _unround)
+    : subtotal_posts(_handler, amount_expr), report(_report), unround(_unround) {
     create_accounts();
     TRACE_CTOR(posts_as_equity, "post_handler_ptr, expr_t&");
   }
@@ -1072,5 +1087,3 @@ public:
 };
 
 } // namespace ledger
-
-#endif // _FILTERS_H
