@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2014, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -39,8 +39,7 @@
  *
  * @ingroup data
  */
-#ifndef _ITEM_H
-#define _ITEM_H
+#pragma once
 
 #include "scope.h"
 
@@ -48,12 +47,12 @@ namespace ledger {
 
 struct position_t
 {
-  path             pathname;
-  istream_pos_type beg_pos;
-  std::size_t      beg_line;
-  istream_pos_type end_pos;
-  std::size_t      end_line;
-  std::size_t      sequence;
+  path                   pathname;
+  std::istream::pos_type beg_pos;
+  std::size_t            beg_line;
+  std::istream::pos_type end_pos;
+  std::size_t            end_line;
+  std::size_t            sequence;
 
   position_t()
     : beg_pos(0), beg_line(0), end_pos(0), end_line(0), sequence(0) {
@@ -78,38 +77,22 @@ struct position_t
     }
     return *this;
   }
-
-#if HAVE_BOOST_SERIALIZATION
-private:
-  /** Serialization. */
-
-  friend class boost::serialization::access;
-
-  template<class Archive>
-  void serialize(Archive& ar, const unsigned int /* version */) {
-    ar & pathname;
-    ar & beg_pos;
-    ar & beg_line;
-    ar & end_pos;
-    ar & end_line;
-    ar & sequence;
-  }
-#endif // HAVE_BOOST_SERIALIZATION
 };
 
-class item_t : public supports_flags<uint_least16_t>, public scope_t
+class item_t : public flags::supports_flags<uint_least16_t>, public scope_t
 {
 public:
 #define ITEM_NORMAL            0x00 // no flags at all, a basic posting
 #define ITEM_GENERATED         0x01 // posting was not found in a journal
 #define ITEM_TEMP              0x02 // posting is a managed temporary
 #define ITEM_NOTE_ON_NEXT_LINE 0x04 // did we see a note on the next line?
+#define ITEM_INFERRED          0x08 // bucketed item
 
   enum state_t { UNCLEARED = 0, CLEARED, PENDING };
 
   typedef std::pair<optional<value_t>, bool> tag_data_t;
   typedef std::map<string, tag_data_t,
-                   function<bool(string, string)> > string_map;
+                   std::function<bool(string, string)> > string_map;
 
   state_t              _state;
   optional<date_t>     _date;
@@ -191,11 +174,7 @@ public:
   static bool use_aux_date;
 
   virtual bool has_date() const {
-#if BOOST_VERSION >= 105600
-    return _date != NULL;
-#else
-    return _date;
-#endif
+    return static_cast<bool>(_date);
   }
 
   virtual date_t date() const {
@@ -226,25 +205,6 @@ public:
                                   const string& name);
 
   bool valid() const;
-
-#if HAVE_BOOST_SERIALIZATION
-private:
-  /** Serialization. */
-
-  friend class boost::serialization::access;
-
-  template<class Archive>
-  void serialize(Archive& ar, const unsigned int /* version */) {
-    ar & boost::serialization::base_object<supports_flags<uint_least16_t> >(*this);
-    ar & boost::serialization::base_object<scope_t>(*this);
-    ar & _state;
-    ar & _date;
-    ar & _date_aux;
-    ar & note;
-    ar & pos;
-    ar & metadata;
-  }
-#endif // HAVE_BOOST_SERIALIZATION
 };
 
 value_t get_comment(item_t& item);
@@ -254,5 +214,3 @@ string  item_context(const item_t& item, const string& desc);
 void    put_metadata(property_tree::ptree& pt, const item_t::string_map& metadata);
 
 } // namespace ledger
-
-#endif // _ITEM_H
