@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -145,6 +145,11 @@ account_t * journal_t::register_account(const string& name, post_t * post,
       if (! post) {
         result->add_flags(ACCOUNT_KNOWN);
       }
+      // If the post is generated by an automatic transaction, mark the
+      // account as known to avoid pedantic errors
+      else if (post->has_flags(ITEM_GENERATED)) {
+        result->add_flags(ACCOUNT_KNOWN);
+      }
       else if (checking_style == CHECK_WARNING) {
         current_context->warning(_f("Unknown account '%1%'") % result->fullname());
       }
@@ -297,7 +302,7 @@ void journal_t::register_metadata(const string& key, const value_t& value,
     }
   }
 
-  if (! value.is_null()) {
+  if (! value.is_null() && context.which() != 0) {
     std::pair<tag_check_exprs_map::iterator,
               tag_check_exprs_map::iterator> range =
       tag_check_exprs.equal_range(key);
@@ -333,12 +338,13 @@ namespace {
     xact_t * xact = context.which() == 1 ? boost::get<xact_t *>(context) : NULL;
     post_t * post = context.which() == 2 ? boost::get<post_t *>(context) : NULL;
 
-    if ((xact || post) && xact ? xact->metadata : post->metadata) {
+    if ((xact || post) && (xact ? xact->metadata : post->metadata)) {
       foreach (const item_t::string_map::value_type& pair,
                xact ? *xact->metadata : *post->metadata) {
         const string& key(pair.first);
 
-        if (optional<value_t> value = pair.second.first)
+        const optional<value_t>& value = pair.second.first;
+        if (value)
           journal.register_metadata(key, *value, context);
         else
           journal.register_metadata(key, NULL_VALUE, context);
@@ -466,7 +472,8 @@ bool journal_t::remove_xact(xact_t * xact)
   return true;
 }
 
-std::size_t journal_t::read(parse_context_stack_t& context)
+std::size_t journal_t::read(parse_context_stack_t& context,
+                            hash_type_t hash_type)
 {
   std::size_t count = 0;
   try {
@@ -485,7 +492,7 @@ std::size_t journal_t::read(parse_context_stack_t& context)
     if (! current.master)
       current.master = master;
 
-    count = read_textual(context);
+    count = read_textual(context, hash_type);
     if (count > 0) {
       if (! current.pathname.empty())
         sources.push_back(fileinfo_t(current.pathname));
