@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -43,8 +43,7 @@
  *
  * Long.
  */
-#ifndef INCLUDED_POOL_H
-#define INCLUDED_POOL_H
+#pragma once
 
 #include "history.h"
 #include "annotate.h"
@@ -135,5 +134,3 @@ public:
 };
 
 } // namespace ledger
-
-#endif // INCLUDED_POOL_H
