@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2014, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -39,8 +39,7 @@
  *
  * @ingroup data
  */
-#ifndef _POST_H
-#define _POST_H
+#pragma once
 
 #include "item.h"
 
@@ -61,6 +60,7 @@ public:
 #define POST_COST_VIRTUAL    0x0400 // cost is virtualized: (@)
 #define POST_ANONYMIZED      0x0800 // a temporary, anonymous posting
 #define POST_DEFERRED        0x1000 // the account was specified with <angles>
+#define POST_IS_TIMELOG      0x2000 // the posting is a timelog entry
 
   xact_t *    xact;             // only set for posts of regular xacts
   account_t * account;
@@ -73,6 +73,12 @@ public:
   optional<datetime_t> checkin;
   optional<datetime_t> checkout;
 
+private:
+
+  optional<string> _payee;
+
+public:
+
   post_t(account_t * _account = NULL,
          flags_t     _flags   = ITEM_NORMAL)
     : item_t(_flags), xact(NULL), account(_account)
@@ -132,10 +138,14 @@ public:
   virtual date_t primary_date() const;
   virtual optional<date_t> aux_date() const;
 
+  string payee_from_tag() const;
   string payee() const;
+  void set_payee(const string& payee) {
+    _payee = payee;
+  }
 
   bool must_balance() const {
-    return ! has_flags(POST_VIRTUAL) || has_flags(POST_MUST_BALANCE);
+    return ! (has_flags(POST_VIRTUAL) || has_flags(POST_IS_TIMELOG)) || has_flags(POST_MUST_BALANCE);
   }
 
   virtual expr_t::ptr_op_t lookup(const symbol_t::kind_t kind,
@@ -205,11 +215,7 @@ public:
   mutable optional<xdata_t> xdata_;
 
   bool has_xdata() const {
-#if BOOST_VERSION >= 105600
-    return xdata_ != NULL;
-#else
-    return xdata_;
-#endif
+    return static_cast<bool>(xdata_);
   }
   void clear_xdata() {
     xdata_ = none;
@@ -255,24 +261,6 @@ public:
       }
     }
   };
-
-#if HAVE_BOOST_SERIALIZATION
-private:
-  /** Serialization. */
-
-  friend class boost::serialization::access;
-
-  template<class Archive>
-  void serialize(Archive& ar, const unsigned int /* version */) {
-    ar & boost::serialization::base_object<item_t>(*this);
-    ar & xact;
-    ar & account;
-    ar & amount;
-    ar & amount_expr;
-    ar & cost;
-    ar & assigned_amount;
-  }
-#endif // HAVE_BOOST_SERIALIZATION
 };
 
 class journal_t;
@@ -280,5 +268,3 @@ void extend_post(post_t& post, journal_t& journal);
 void put_post(property_tree::ptree& pt, const post_t& post);
 
 } // namespace ledger
-
-#endif // _POST_H
