@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -74,7 +74,7 @@ namespace {
         parsing_context.get_current().journal = report.session.journal.get();
         parsing_context.get_current().scope   = &report.session;
 
-        report.session.journal->read(parsing_context);
+        report.session.journal->read(parsing_context, NO_HASHES);
         report.session.journal->clear_xdata();
       }
     }
