@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -39,6 +39,7 @@
 
 namespace ledger {
 
+using namespace python;
 using namespace boost::python;
 
 namespace {
@@ -206,6 +207,10 @@ internal precision."))
     .def("in_place_round", &amount_t::in_place_round,
          return_internal_reference<>())
 
+    .def("__round__", &amount_t::roundto)
+    .def("in_place_roundto", &amount_t::in_place_roundto,
+         return_internal_reference<>())
+
     .def("truncated", &amount_t::truncated)
     .def("in_place_truncate", &amount_t::in_place_truncate,
          return_internal_reference<>())
