@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -36,6 +36,8 @@
 
 namespace ledger {
 
+using namespace flags;
+using namespace python;
 using namespace boost::python;
 
 struct bool_to_python
@@ -73,7 +75,6 @@ struct bool_from_python
 typedef register_python_conversion<bool, bool_to_python, bool_from_python>
   bool_python_conversion;
 
-
 struct string_to_python
 {
   static PyObject* convert(const string& str)
@@ -102,7 +103,6 @@ struct string_from_python
       return;
 #endif
 
-    const Py_UNICODE* value;
     Py_ssize_t size;
     string str;
 
@@ -133,9 +133,9 @@ struct string_from_python
       default:
         assert("PyUnicode_KIND returned an unexpected kind" == NULL);
     }
-#else
+#else // PY_MINOR_VERSION >= 3
     size = PyUnicode_GET_SIZE(obj_ptr);
-    value = PyUnicode_AS_UNICODE(obj_ptr);
+    const Py_UNICODE* value = PyUnicode_AS_UNICODE(obj_ptr);
 #if Py_UNICODE_SIZE == 2 // UTF-16
     utf8::unchecked::utf16to8(value, value + size, std::back_inserter(str));
 #elif Py_UNICODE_SIZE == 4 // UTF-32
@@ -143,7 +143,7 @@ struct string_from_python
 #else
     assert("Py_UNICODE has an unexpected size" == NULL);
 #endif
-#endif
+#endif // PY_MINOR_VERSION >= 3
 
     void* storage =
       reinterpret_cast<converter::rvalue_from_python_storage<string> *>
