@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2016, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -39,8 +39,7 @@
  *
  * @ingroup report
  */
-#ifndef _SESSION_H
-#define _SESSION_H
+#pragma once
 
 #include "account.h"
 #include "journal.h"
@@ -52,6 +51,16 @@ namespace ledger {
 
 class xact_t;
 
+struct ComparePaths
+{
+  bool operator()(const path& p1, const path& p2) const
+  {
+    return p1 < p2 && !boost::filesystem::equivalent(p1, p2);
+  }
+};
+
+#define COMMA ,
+
 class session_t : public symbol_scope_t
 {
   friend void set_session_context(session_t * session);
@@ -103,6 +112,7 @@ public:
     HANDLER(decimal_comma).report(out);
     HANDLER(time_colon).report(out);
     HANDLER(file_).report(out);
+    HANDLER(hashes_).report(out);
     HANDLER(input_date_format_).report(out);
     HANDLER(explicit).report(out);
     HANDLER(master_account_).report(out);
@@ -153,6 +163,20 @@ public:
      data_files.push_back(str);
    });
 
+  OPTION__
+  (session_t, hashes_,
+   hash_type_t hash_type = NO_HASHES;
+   CTOR(session_t, hashes_) {}
+   DO_(str) {
+     if (str == "sha512" || str == "SHA512") {
+       hash_type = HASH_SHA512;
+     } else if (str == "sha512_half" || str == "SHA512_Half") {
+       hash_type = HASH_SHA512_Half;
+     } else {
+        throw_(std::invalid_argument, _f("Unrecognized hash type"));
+     }
+   });
+
   OPTION_(session_t, input_date_format_, DO_(str) {
       // This changes static variables inside times.h, which affects the
       // basic date parser.
@@ -181,5 +205,3 @@ public:
 void set_session_context(session_t * session);
 
 } // namespace ledger
-
-#endif // _SESSION_H
