@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -40,13 +40,14 @@
 #include "query.h"
 #include "pstream.h"
 #include "pool.h"
+#include "session.h"
 #include <algorithm>
 #if HAVE_BOOST_PYTHON
 #include "pyinterp.h"
 #endif
 
 #define TIMELOG_SUPPORT 1
-#if defined(TIMELOG_SUPPORT)
+#if TIMELOG_SUPPORT
 #include "timelog.h"
 #endif
 
@@ -79,17 +80,20 @@ namespace {
     instance_t *             parent;
     std::list<application_t> apply_stack;
     bool                     no_assertions;
-#if defined(TIMELOG_SUPPORT)
+    hash_type_t              hash_type;
+#if TIMELOG_SUPPORT
     time_log_t               timelog;
 #endif
 
     instance_t(parse_context_stack_t& _context_stack,
                parse_context_t&       _context,
                instance_t *           _parent = NULL,
-               const bool             _no_assertions = false)
+               const bool             _no_assertions = false,
+               const hash_type_t      _hash_type = NO_HASHES)
       : context_stack(_context_stack), context(_context),
         in(*context.stream.get()), parent(_parent),
-        no_assertions(_no_assertions), timelog(context) {}
+        no_assertions(_no_assertions), hash_type(_hash_type),
+        timelog(context) {}
 
     virtual string description() {
       return _("textual parser");
@@ -136,9 +140,9 @@ namespace {
     }
 #endif
 
-    void read_next_directive(bool& error_flag);
+    xact_t * read_next_directive(bool& error_flag, xact_t * previous_xact);
 
-#if defined(TIMELOG_SUPPORT)
+#if TIMELOG_SUPPORT
     void clock_in_directive(char * line, bool capitalized);
     void clock_out_directive(char * line, bool capitalized);
 #endif
@@ -173,10 +177,11 @@ namespace {
     void apply_account_directive(char * line);
     void apply_tag_directive(char * line);
     void apply_rate_directive(char * line);
-    void apply_year_directive(char * line);
+    void apply_year_directive(char * line, bool use_apply_stack = false);
     void end_apply_directive(char * line);
 
-    void xact_directive(char * line, std::streamsize len);
+    xact_t * xact_directive(char * line, std::streamsize len,
+                            xact_t * previous_xact);
     void period_xact_directive(char * line);
     void automated_xact_directive(char * line);
     void price_xact_directive(char * line);
@@ -207,7 +212,8 @@ namespace {
 
     xact_t * parse_xact(char *          line,
                         std::streamsize len,
-                        account_t *     account);
+                        account_t *     account,
+                        xact_t *        previous_xact);
 
     virtual expr_t::ptr_op_t lookup(const symbol_t::kind_t kind,
                                     const string& name);
@@ -247,10 +253,13 @@ void instance_t::parse()
   context.curr_pos = in.tellg();
 
   bool error_flag = false;
+  xact_t * previous_xact = NULL;
 
-  while (in.good() && ! in.eof()) {
+  while (in.good() && ! in.eof() && in.peek() != '^' && in.good()) {
     try {
-      read_next_directive(error_flag);
+      if (xact_t * xact = read_next_directive(error_flag, previous_xact)) {
+        previous_xact = xact;
+      }
     }
     catch (const std::exception& err) {
       error_flag = true;
@@ -295,7 +304,7 @@ void instance_t::parse()
 
   apply_stack.pop_front();
 
-#if defined(TIMELOG_SUPPORT)
+#if TIMELOG_SUPPORT
   timelog.close();
 #endif // TIMELOG_SUPPORT
 
@@ -340,7 +349,8 @@ std::streamsize instance_t::read_line(char *& line)
         --len;
     }
 
-    while (len > 0 && std::isspace(line[len - 1])) // strip trailing whitespace
+    // strip trailing whitespace
+    while (len > 0 && std::isspace(static_cast<unsigned char>(line[len - 1])))
       line[--len] = '\0';
 
     return len;
@@ -348,14 +358,14 @@ std::streamsize instance_t::read_line(char *& line)
   return 0;
 }
 
-void instance_t::read_next_directive(bool& error_flag)
+xact_t * instance_t::read_next_directive(bool& error_flag, xact_t * previous_xact)
 {
   char * line;
   std::streamsize len = read_line(line);
   if (len == 0 || line == NULL)
-    return;
+    return NULL;
 
-  if (! std::isspace(line[0]))
+  if (! std::isspace(static_cast<unsigned char>(line[0])))
     error_flag = false;
 
   switch (line[0]) {
@@ -389,8 +399,7 @@ void instance_t::read_next_directive(bool& error_flag)
   case '7':
   case '8':
   case '9':
-    xact_directive(line, len);
-    break;
+    return xact_directive(line, len, previous_xact);
   case '=':                     // automated xact
     automated_xact_directive(line);
     break;
@@ -405,7 +414,7 @@ void instance_t::read_next_directive(bool& error_flag)
   default:                      // some other directive
     if (! general_directive(line)) {
       switch (line[0]) {
-#if defined(TIMELOG_SUPPORT)
+#if TIMELOG_SUPPORT
       case 'i':
         clock_in_directive(line, false);
         break;
@@ -449,9 +458,11 @@ void instance_t::read_next_directive(bool& error_flag)
     }
     break;
   }
+
+  return NULL;
 }
 
-#if defined(TIMELOG_SUPPORT)
+#if TIMELOG_SUPPORT
 
 void instance_t::clock_in_directive(char * line, bool capitalized)
 {
@@ -615,12 +626,16 @@ void instance_t::automated_xact_directive(char * line)
         item->pos->end_line++;
       }
       else if ((remlen > 7 && *p == 'a' &&
-                std::strncmp(p, "assert", 6) == 0 && std::isspace(p[6])) ||
+                std::strncmp(p, "assert", 6) == 0 &&
+                std::isspace(static_cast<unsigned char>(p[6]))) ||
                (remlen > 6 && *p == 'c' &&
-                std::strncmp(p, "check", 5) == 0 && std::isspace(p[5])) ||
+                std::strncmp(p, "check", 5) == 0 &&
+                std::isspace(static_cast<unsigned char>(p[5]))) ||
                (remlen > 5 && *p == 'e' &&
-                ((std::strncmp(p, "expr", 4) == 0 && std::isspace(p[4])) ||
-                 (std::strncmp(p, "eval", 4) == 0 && std::isspace(p[4]))))) {
+                ((std::strncmp(p, "expr", 4) == 0 &&
+                  std::isspace(static_cast<unsigned char>(p[4]))) ||
+                 (std::strncmp(p, "eval", 4) == 0 &&
+                  std::isspace(static_cast<unsigned char>(p[4])))))) {
         const char c = *p;
         p = skip_ws(&p[*p == 'a' ? 6 : (*p == 'c' ? 5 : 4)]);
         if (! ae->check_exprs)
@@ -711,16 +726,17 @@ void instance_t::period_xact_directive(char * line)
   }
 }
 
-void instance_t::xact_directive(char * line, std::streamsize len)
+xact_t * instance_t::xact_directive(char * line, std::streamsize len,
+                                    xact_t * previous_xact)
 {
   TRACE_START(xacts, 1, "Time spent handling transactions:");
 
-  if (xact_t * xact = parse_xact(line, len, top_account())) {
+  if (xact_t * xact = parse_xact(line, len, top_account(), previous_xact)) {
     unique_ptr<xact_t> manager(xact);
 
     if (context.journal->add_xact(xact)) {
-      manager.release();        // it's owned by the journal now
       context.count++;
+      return manager.release(); // it's owned by the journal now
     }
     // It's perfectly valid for the journal to reject the xact, which it
     // will do if the xact has no substantive effect (for example, a
@@ -730,6 +746,8 @@ void instance_t::xact_directive(char * line, std::streamsize len)
   }
 
   TRACE_STOP(xacts, 1);
+
+  return NULL;
 }
 
 void instance_t::include_directive(char * line)
@@ -775,6 +793,15 @@ void instance_t::include_directive(char * line)
       if (is_regular_file(*iter))
         {
         string base = (*iter).filename().string();
+        // Skip files with invalid UTF-8 in their names to avoid encoding errors
+        if (!utf8::is_valid(base.begin(), base.end())) {
+          DEBUG("textual.include", "Skipping file with invalid UTF-8 name: " << base);
+          continue;
+        }
+        if (context.pathname == *iter) {
+          DEBUG("textual.include", "Avoiding recursive include of: " << *iter);
+          continue;
+        }
         if (glob.match(base)) {
           journal_t *  journal  = context.journal;
           account_t *  master   = top_account();
@@ -791,9 +818,13 @@ void instance_t::include_directive(char * line)
           context_stack.get_current().journal = journal;
           context_stack.get_current().master  = master;
           context_stack.get_current().scope   = scope;
+          
+          parse_context_t * save_current_context = journal->current_context;
+          journal->current_context = &context_stack.get_current();
+          
           try {
             instance_t instance(context_stack, context_stack.get_current(),
-                                this, no_assertions);
+                                this, no_assertions, hash_type);
             instance.apply_stack.push_front(application_t("account", master));
             instance.parse();
           }
@@ -802,6 +833,7 @@ void instance_t::include_directive(char * line)
             count    += context_stack.get_current().count;
             sequence += context_stack.get_current().sequence;
 
+            journal->current_context = save_current_context;
             context_stack.pop();
             throw;
           }
@@ -810,6 +842,7 @@ void instance_t::include_directive(char * line)
           count    += context_stack.get_current().count;
           sequence += context_stack.get_current().sequence;
 
+          journal->current_context = save_current_context;
           context_stack.pop();
 
           files_found = true;
@@ -837,17 +870,15 @@ void instance_t::apply_directive(char * line)
   else if (keyword == "fixed" || keyword == "rate")
     apply_rate_directive(b);
   else if (keyword == "year")
-    apply_year_directive(b);
+    apply_year_directive(b, true);  // "apply year" uses apply_stack
 }
 
 void instance_t::apply_account_directive(char * line)
 {
   if (account_t * acct = top_account()->find_account(line))
     apply_stack.push_front(application_t("account", acct));
-#if !NO_ASSERTS
   else
     assert("Failed to create account" == NULL);
-#endif
 }
 
 void instance_t::apply_tag_directive(char * line)
@@ -873,16 +904,27 @@ void instance_t::apply_rate_directive(char * line)
   }
 }
 
-void instance_t::apply_year_directive(char * line)
+void instance_t::apply_year_directive(char * line, bool use_apply_stack)
 {
   try {
     unsigned short year(lexical_cast<unsigned short>(skip_ws(line)));
-    apply_stack.push_front(application_t("year", epoch));
+    if (use_apply_stack) {
+      // Used for "apply year" which needs "end apply"
+      apply_stack.push_front(application_t("year", epoch));
+    }
+    // Otherwise for plain "year" directive, don't use apply_stack - it's a permanent change
     DEBUG("times.epoch", "Setting current year to " << year);
-    // This must be set to the last day of the year, otherwise partial
-    // dates like "11/01" will refer to last year's November, not the
-    // current year.
-    epoch = datetime_t(date_t(year, 12, 31));
+    
+    // Track the year directive separately
+    year_directive_year = year;
+    
+    // Only set epoch if it's not already set (e.g., by --now)
+    if (!epoch) {
+      // This must be set to the last day of the year, otherwise partial
+      // dates like "11/01" will refer to last year's November, not the
+      // current year.
+      epoch = datetime_t(date_t(year, 12, 31));
+    }
   } catch(bad_lexical_cast &) {
     throw_(parse_error, _f("Argument '%1%' not a valid year") % skip_ws(line));
   }
@@ -1015,7 +1057,7 @@ void instance_t::alias_directive(char * line)
 {
   if (char * e = std::strchr(line, '=')) {
     char * z = e - 1;
-    while (std::isspace(*z))
+    while (std::isspace(static_cast<unsigned char>(*z)))
       *z-- = '\0';
     *e++ = '\0';
     e = skip_ws(e);
@@ -1234,7 +1276,7 @@ void instance_t::python_directive(char * line)
     if (read_line(line) > 0) {
       if (! indent) {
         const char * p = line;
-        while (*p && std::isspace(*p)) {
+        while (*p && std::isspace(static_cast<unsigned char>(*p))) {
           ++indent;
           ++p;
         }
@@ -1242,7 +1284,7 @@ void instance_t::python_directive(char * line)
 
       const char * p = line;
       for (std::size_t i = 0; i < indent; i++) {
-        if (std::isspace(*p))
+        if (std::isspace(static_cast<unsigned char>(*p)))
           ++p;
         else
           break;
@@ -1257,7 +1299,7 @@ void instance_t::python_directive(char * line)
     python_session->initialize();
 
   python_session->main_module->define_global
-    ("journal", python::object(python::ptr(context.journal)));
+    ("journal", boost::python::object(boost::python::ptr(context.journal)));
   python_session->eval(script.str(), python_interpreter_t::PY_EVAL_MULTI);
 }
 
@@ -1473,7 +1515,7 @@ post_t * instance_t::parse_post(char *          line,
   char * next = next_element(p, true);
   char * e = p + std::strlen(p);
 
-  while (e > p && std::isspace(*(e - 1)))
+  while (e > p && std::isspace(static_cast<unsigned char>(*(e - 1))))
     e--;
 
   if ((*p == '[' && *(e - 1) == ']') || (*p == '(' && *(e - 1) == ')')) {
@@ -1640,7 +1682,7 @@ post_t * instance_t::parse_post(char *          line,
       ptristream stream(p, static_cast<std::size_t>(len - beg));
 
       if (*p != '(')            // indicates a value expression
-        post->assigned_amount->parse(stream, PARSE_NO_MIGRATE);
+        post->assigned_amount->parse(stream);
       else
         parse_amount_expr(stream, *context.scope, *post.get(),
                           *post->assigned_amount,
@@ -1658,7 +1700,7 @@ post_t * instance_t::parse_post(char *          line,
 
       const amount_t& amt(*post->assigned_amount);
       value_t account_total
-        (post->account->amount(!post->has_flags(POST_VIRTUAL)).strip_annotations(keep_details_t()));
+        (post->account->amount(!(post->has_flags(POST_VIRTUAL) || post->has_flags(POST_IS_TIMELOG))).strip_annotations(keep_details_t()));
 
       DEBUG("post.assign", "line " << context.linenum << ": "
             << "account balance = " << account_total);
@@ -1693,7 +1735,9 @@ post_t * instance_t::parse_post(char *          line,
 
       // Subtract amounts from previous posts to this account in the xact.
       for (post_t* p : xact->posts) {
-        if (p->account == post->account && p->has_flags(POST_VIRTUAL) == post->has_flags(POST_VIRTUAL)) {
+        if (p->account == post->account && 
+            ((p->has_flags(POST_VIRTUAL) || p->has_flags(POST_IS_TIMELOG)) == 
+             (post->has_flags(POST_VIRTUAL) || post->has_flags(POST_IS_TIMELOG)))) {
           amount_t amt(p->amount.strip_annotations(keep_details_t()));
           diff -= amt;
           DEBUG("textual.parse", "line " << context.linenum << ": "
@@ -1818,7 +1862,8 @@ bool instance_t::parse_posts(account_t *  account,
 
 xact_t * instance_t::parse_xact(char *          line,
                                 std::streamsize len,
-                                account_t *     account)
+                                account_t *     account,
+                                xact_t *        previous_xact)
 {
   TRACE_START(xact_text, 1, "Time spent parsing transaction text:");
 
@@ -1884,7 +1929,7 @@ xact_t * instance_t::parse_xact(char *          line,
       }
       else if (*p == ';' && (tabs > 0 || spaces > 1)) {
         char *q = p - 1;
-        while (q > next && std::isspace(*q))
+        while (q > next && std::isspace(static_cast<unsigned char>(*q)))
           --q;
         if (q >= next)
           *(q + 1) = '\0';
@@ -1937,11 +1982,14 @@ xact_t * instance_t::parse_xact(char *          line,
       item->pos->end_line++;
     }
     else if ((remlen > 7 && *p == 'a' &&
-              std::strncmp(p, "assert", 6) == 0 && std::isspace(p[6])) ||
+              std::strncmp(p, "assert", 6) == 0 &&
+              std::isspace(static_cast<unsigned char>(p[6]))) ||
              (remlen > 6 && *p == 'c' &&
-              std::strncmp(p, "check", 5) == 0 && std::isspace(p[5])) ||
+              std::strncmp(p, "check", 5) == 0 &&
+              std::isspace(static_cast<unsigned char>(p[5]))) ||
              (remlen > 5 && *p == 'e' &&
-              std::strncmp(p, "expr", 4) == 0 && std::isspace(p[4]))) {
+              std::strncmp(p, "expr", 4) == 0 &&
+              std::isspace(static_cast<unsigned char>(p[4])))) {
       const char c = *p;
       p = skip_ws(&p[*p == 'a' ? 6 : (*p == 'c' ? 5 : 4)]);
       expr_t expr(p);
@@ -2007,6 +2055,25 @@ xact_t * instance_t::parse_xact(char *          line,
 
   TRACE_STOP(xact_details, 1);
 
+  if (hash_type != NO_HASHES) {
+    string expected_hash =
+      xact->hash(previous_xact &&
+                 previous_xact->has_tag("Hash") ?
+                 previous_xact->get_tag("Hash")->to_string() : "",
+                 hash_type);
+    if (xact->has_tag("Hash")) {
+      string current_hash = xact->get_tag("Hash")->to_string();
+      if (! std::equal(expected_hash.begin(),
+                       expected_hash.begin() +
+                       std::min(expected_hash.size(), current_hash.size()),
+                       current_hash.begin()))
+        throw_(parse_error, _f("Expected hash %1% != %2%") %
+               expected_hash % current_hash);
+    } else {
+      xact->set_tag("Hash", string_value(expected_hash));
+    }
+  }
+
   return xact.release();
 
   }
@@ -2027,12 +2094,14 @@ expr_t::ptr_op_t instance_t::lookup(const symbol_t::kind_t kind,
   return context.scope->lookup(kind, name);
 }
 
-std::size_t journal_t::read_textual(parse_context_stack_t& context_stack)
+std::size_t journal_t::read_textual(parse_context_stack_t& context_stack,
+                                    hash_type_t hash_type)
 {
   TRACE_START(parsing_total, 1, "Total time spent parsing text:");
   {
     instance_t instance(context_stack, context_stack.get_current(), NULL,
-                        checking_style == journal_t::CHECK_PERMISSIVE);
+                        checking_style == journal_t::CHECK_PERMISSIVE,
+                        hash_type);
     instance.apply_stack.push_front
       (application_t("account", context_stack.get_current().master));
     instance.parse();
