@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2019, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -40,6 +40,7 @@
 namespace ledger {
 
 optional<datetime_t> epoch;
+optional<int> year_directive_year;  // Track year from year directives separately
 
 date_time::weekdays start_of_week = gregorian::Sunday;
 
@@ -55,18 +56,18 @@ namespace {
 
     temporal_io_t(const char * _fmt_str, bool _input)
       : fmt_str(_fmt_str),
-        traits(icontains(fmt_str, "%y"),
-               icontains(fmt_str, "%m") || icontains(fmt_str, "%b"),
-               icontains(fmt_str, "%d")),
+        traits(icontains(fmt_str, "%F") || icontains(fmt_str, "%y"),
+               icontains(fmt_str, "%F") || icontains(fmt_str, "%m") || icontains(fmt_str, "%b"),
+               icontains(fmt_str, "%F") || icontains(fmt_str, "%d")),
         input(_input) {
     }
 
     void set_format(const char * fmt) {
       fmt_str  = fmt;
-      traits   = date_traits_t(icontains(fmt_str, "%y"),
-                               icontains(fmt_str, "%m") ||
-                               icontains(fmt_str, "%b"),
-                               icontains(fmt_str, "%d"));
+      traits   = date_traits_t(icontains(fmt_str, "%F") || icontains(fmt_str, "%y"),
+                               icontains(fmt_str, "%F") ||
+                               icontains(fmt_str, "%m") || icontains(fmt_str, "%b"),
+                               icontains(fmt_str, "%F") || icontains(fmt_str, "%d"));
     }
 
     T parse(const char *) {}
@@ -99,7 +100,13 @@ namespace {
   {
     std::tm data;
     std::memset(&data, 0, sizeof(std::tm));
-    data.tm_year = CURRENT_DATE().year() - 1900;
+    // When year_directive_year is set (from "apply year" or "year" directive),
+    // use that instead of CURRENT_DATE() to avoid issues with leap year dates
+    if (year_directive_year) {
+      data.tm_year = *year_directive_year - 1900;
+    } else {
+      data.tm_year = CURRENT_DATE().year() - 1900;
+    }
     data.tm_mday = 1;           // some formats have no day
     if (strptime(str, fmt_str.c_str(), &data))
       return gregorian::date_from_tm(data);
@@ -114,7 +121,6 @@ namespace {
 
   shared_ptr<datetime_io_t> input_datetime_io;
   shared_ptr<datetime_io_t> timelog_datetime_io;
-  shared_ptr<date_io_t>     input_date_io;
   shared_ptr<datetime_io_t> written_datetime_io;
   shared_ptr<date_io_t>     written_date_io;
   shared_ptr<datetime_io_t> printed_datetime_io;
@@ -163,10 +169,30 @@ namespace {
         *traits = io.traits;
 
       if (! io.traits.has_year) {
-        when = date_t(CURRENT_DATE().year(), when.month(), when.day());
-
-        if (when.month() > CURRENT_DATE().month())
-          when -= gregorian::years(1);
+        // First check if we have a year directive year to use
+        if (year_directive_year) {
+          // Use the year from the year directive
+          when = date_t(*year_directive_year, when.month(), when.day());
+          DEBUG("times.parse", "Using year directive year: " << *year_directive_year);
+        } else if (epoch) {
+          // When using the epoch (e.g., from --now), use it for the year
+          date_t reference_date = epoch->date();
+          when = date_t(reference_date.year(), when.month(), when.day());
+          
+          // Apply month rollback if the parsed month is after the epoch's month
+          if (when.month() > reference_date.month())
+            when -= gregorian::years(1);
+          
+          DEBUG("times.parse", "Using epoch year: " << reference_date.year() 
+                << ", epoch month: " << reference_date.month()
+                << ", parsed month: " << when.month()
+                << ", final year: " << when.year());
+        } else {
+          // When no epoch, use current date and handle month rollback
+          when = date_t(CURRENT_DATE().year(), when.month(), when.day());
+          if (when.month() > CURRENT_DATE().month())
+            when -= gregorian::years(1);
+        }
       }
     }
     return when;
@@ -174,13 +200,6 @@ namespace {
 
   date_t parse_date_mask(const char * date_str, date_traits_t * traits = NULL)
   {
-    if (input_date_io.get()) {
-      date_t when = parse_date_mask_routine(date_str, *input_date_io.get(),
-                                            traits);
-      if (! when.is_not_a_date())
-        return when;
-    }
-
     foreach (shared_ptr<date_io_t>& reader, readers) {
       date_t when = parse_date_mask_routine(date_str, *reader.get(), traits);
       if (! when.is_not_a_date())
@@ -245,6 +264,10 @@ string_to_month_of_year(const std::string& str)
 
 datetime_t parse_datetime(const char * str)
 {
+  if (std::strlen(str) > 127) {
+    throw_(date_error, _f("Invalid date: %1%") % str);
+  }
+
   char buf[128];
   std::strcpy(buf, str);
 
@@ -273,12 +296,10 @@ date_t date_specifier_t::begin() const
   month_type the_month = month ? *month : date_t::month_type(1);
   day_type   the_day   = day   ? *day   : date_t::day_type(1);
 
-#if !NO_ASSERTS
   if (day)
     assert(! wday);
   else if (wday)
     assert(! day);
-#endif
 
   // jww (2009-11-16): Handle wday.  If a month is set, find the most recent
   // wday in that month; if the year is set, then in that year.
@@ -572,6 +593,39 @@ public:
 
   date_interval_t parse();
 
+private:
+  // Helper methods for parsing different token types
+  void handle_date_token(lexer_t::token_t& tok,
+                         optional<date_specifier_t>& inclusion_specifier);
+  void handle_dash_operator(lexer_t::token_t& tok,
+                            optional<date_specifier_t>& inclusion_specifier,
+                            optional<date_specifier_t>& since_specifier,
+                            optional<date_specifier_t>& until_specifier,
+                            bool& end_inclusive);
+  void handle_since_token(lexer_t::token_t& tok,
+                         optional<date_specifier_t>& since_specifier);
+  void handle_until_token(lexer_t::token_t& tok,
+                         optional<date_specifier_t>& until_specifier);
+  void handle_in_token(lexer_t::token_t& tok,
+                      optional<date_specifier_t>& inclusion_specifier);
+  void handle_relative_token(lexer_t::token_t& tok,
+                             optional<date_specifier_t>& since_specifier,
+                             optional<date_specifier_t>& until_specifier,
+                             optional<date_specifier_t>& inclusion_specifier,
+                             const date_t& today);
+  void handle_simple_period_token(lexer_t::token_t::kind_t kind,
+                                  optional<date_specifier_t>& inclusion_specifier,
+                                  const date_t& today);
+  void handle_every_token(lexer_t::token_t& tok,
+                         optional<date_duration_t>& duration);
+  void handle_period_token(lexer_t::token_t::kind_t kind,
+                          optional<date_duration_t>& duration);
+  void finalize_period(date_interval_t& period,
+                      const optional<date_specifier_t>& since_specifier,
+                      const optional<date_specifier_t>& until_specifier,
+                      const optional<date_specifier_t>& inclusion_specifier,
+                      bool end_inclusive);
+
 private:
   void determine_when(lexer_t::token_t& tok, date_specifier_t& specifier);
 };
@@ -791,6 +845,329 @@ void date_parser_t::determine_when(date_parser_t::lexer_t::token_t& tok,
   }
 }
 
+// Helper method implementations
+void date_parser_t::handle_date_token(lexer_t::token_t& tok,
+                                      optional<date_specifier_t>& inclusion_specifier)
+{
+  if (!inclusion_specifier)
+    inclusion_specifier = date_specifier_t();
+  determine_when(tok, *inclusion_specifier);
+}
+
+void date_parser_t::handle_dash_operator(lexer_t::token_t& tok,
+                                         optional<date_specifier_t>& inclusion_specifier,
+                                         optional<date_specifier_t>& since_specifier,
+                                         optional<date_specifier_t>& until_specifier,
+                                         bool& end_inclusive)
+{
+  if (inclusion_specifier) {
+    since_specifier = inclusion_specifier;
+    until_specifier = date_specifier_t();
+    inclusion_specifier = none;
+
+    tok = lexer.next_token();
+    determine_when(tok, *until_specifier);
+
+    // The dash operator is special: it has an _inclusive_ end.
+    end_inclusive = true;
+  } else {
+    tok.unexpected();
+  }
+}
+
+void date_parser_t::handle_since_token(lexer_t::token_t& tok,
+                                       optional<date_specifier_t>& since_specifier)
+{
+  if (since_specifier) {
+    tok.unexpected();
+  } else {
+    since_specifier = date_specifier_t();
+    tok = lexer.next_token();
+    determine_when(tok, *since_specifier);
+  }
+}
+
+void date_parser_t::handle_until_token(lexer_t::token_t& tok,
+                                       optional<date_specifier_t>& until_specifier)
+{
+  if (until_specifier) {
+    tok.unexpected();
+  } else {
+    until_specifier = date_specifier_t();
+    tok = lexer.next_token();
+    determine_when(tok, *until_specifier);
+  }
+}
+
+void date_parser_t::handle_in_token(lexer_t::token_t& tok,
+                                    optional<date_specifier_t>& inclusion_specifier)
+{
+  if (inclusion_specifier) {
+    tok.unexpected();
+  } else {
+    inclusion_specifier = date_specifier_t();
+    tok = lexer.next_token();
+    determine_when(tok, *inclusion_specifier);
+  }
+}
+
+void date_parser_t::handle_relative_token(lexer_t::token_t& tok,
+                                          optional<date_specifier_t>& since_specifier,
+                                          optional<date_specifier_t>& until_specifier,
+                                          optional<date_specifier_t>& inclusion_specifier,
+                                          const date_t& today)
+{
+  int8_t adjust = 0;
+  if (tok.kind == lexer_t::token_t::TOK_NEXT)
+    adjust = 1;
+  else if (tok.kind == lexer_t::token_t::TOK_LAST)
+    adjust = -1;
+
+  tok = lexer.next_token();
+  switch (tok.kind) {
+  case lexer_t::token_t::TOK_INT: {
+    unsigned short amount = boost::get<unsigned short>(*tok.value);
+    date_t base(today);
+    date_t end(today);
+
+    if (!adjust)
+      adjust = 1;
+
+    tok = lexer.next_token();
+    switch (tok.kind) {
+    case lexer_t::token_t::TOK_YEARS:
+      base += gregorian::years(amount * adjust);
+      break;
+    case lexer_t::token_t::TOK_QUARTERS:
+      base += gregorian::months(amount * adjust * 3);
+      break;
+    case lexer_t::token_t::TOK_MONTHS:
+      base += gregorian::months(amount * adjust);
+      break;
+    case lexer_t::token_t::TOK_WEEKS:
+      base += gregorian::weeks(amount * adjust);
+      break;
+    case lexer_t::token_t::TOK_DAYS:
+      base += gregorian::days(amount * adjust);
+      break;
+    default:
+      tok.unexpected();
+      break;
+    }
+
+    if (adjust >= 0) {
+      date_t temp = base;
+      base = end;
+      end = temp;
+    }
+
+    since_specifier = date_specifier_t(base);
+    until_specifier = date_specifier_t(end);
+    break;
+  }
+
+  case lexer_t::token_t::TOK_A_MONTH: {
+    inclusion_specifier = date_specifier_t();
+    determine_when(tok, *inclusion_specifier);
+
+    date_t temp(today.year(), *inclusion_specifier->month, 1);
+    temp += gregorian::years(adjust);
+    inclusion_specifier =
+      date_specifier_t(static_cast<date_specifier_t::year_type>(temp.year()),
+                       temp.month());
+    break;
+  }
+
+  case lexer_t::token_t::TOK_A_WDAY: {
+    inclusion_specifier = date_specifier_t();
+    determine_when(tok, *inclusion_specifier);
+
+    date_t temp = date_duration_t::find_nearest(today, date_duration_t::WEEKS);
+    while (temp.day_of_week() != inclusion_specifier->wday)
+      temp += gregorian::days(1);
+    temp += gregorian::days(7 * adjust);
+    inclusion_specifier = date_specifier_t(temp);
+    break;
+  }
+
+  case lexer_t::token_t::TOK_YEAR: {
+    date_t temp(today);
+    temp += gregorian::years(adjust);
+    inclusion_specifier =
+      date_specifier_t(static_cast<date_specifier_t::year_type>(temp.year()));
+    break;
+  }
+
+  case lexer_t::token_t::TOK_QUARTER: {
+    date_t base = date_duration_t::find_nearest(today, date_duration_t::QUARTERS);
+    date_t temp;
+    if (adjust < 0) {
+      temp = base + gregorian::months(3 * adjust);
+    } else if (adjust == 0) {
+      temp = base + gregorian::months(3);
+    } else if (adjust > 0) {
+      base += gregorian::months(3 * adjust);
+      temp = base + gregorian::months(3 * adjust);
+    }
+    since_specifier = date_specifier_t(adjust < 0 ? temp : base);
+    until_specifier = date_specifier_t(adjust < 0 ? base : temp);
+    break;
+  }
+
+  case lexer_t::token_t::TOK_WEEK: {
+    date_t base = date_duration_t::find_nearest(today, date_duration_t::WEEKS);
+    date_t temp;
+    if (adjust < 0) {
+      temp = base + gregorian::days(7 * adjust);
+    } else if (adjust == 0) {
+      temp = base + gregorian::days(7);
+    } else if (adjust > 0) {
+      base += gregorian::days(7 * adjust);
+      temp = base + gregorian::days(7 * adjust);
+    }
+    since_specifier = date_specifier_t(adjust < 0 ? temp : base);
+    until_specifier = date_specifier_t(adjust < 0 ? base : temp);
+    break;
+  }
+
+  case lexer_t::token_t::TOK_DAY: {
+    date_t temp(today);
+    temp += gregorian::days(adjust);
+    inclusion_specifier = date_specifier_t(temp);
+    break;
+  }
+
+  default:
+  case lexer_t::token_t::TOK_MONTH: {
+    date_t temp(today);
+    temp += gregorian::months(adjust);
+    inclusion_specifier =
+      date_specifier_t(static_cast<date_specifier_t::year_type>(temp.year()),
+                       temp.month());
+    break;
+  }
+  }
+}
+
+void date_parser_t::handle_simple_period_token(lexer_t::token_t::kind_t kind,
+                                               optional<date_specifier_t>& inclusion_specifier,
+                                               const date_t& today)
+{
+  switch (kind) {
+  case lexer_t::token_t::TOK_TODAY:
+    inclusion_specifier = date_specifier_t(today);
+    break;
+  case lexer_t::token_t::TOK_TOMORROW:
+    inclusion_specifier = date_specifier_t(today + gregorian::days(1));
+    break;
+  case lexer_t::token_t::TOK_YESTERDAY:
+    inclusion_specifier = date_specifier_t(today - gregorian::days(1));
+    break;
+  default:
+    break;
+  }
+}
+
+void date_parser_t::handle_every_token(lexer_t::token_t& tok,
+                                       optional<date_duration_t>& duration)
+{
+  tok = lexer.next_token();
+  if (tok.kind == lexer_t::token_t::TOK_INT) {
+    int quantity = boost::get<unsigned short>(*tok.value);
+    tok = lexer.next_token();
+    switch (tok.kind) {
+    case lexer_t::token_t::TOK_YEARS:
+      duration = date_duration_t(date_duration_t::YEARS, quantity);
+      break;
+    case lexer_t::token_t::TOK_QUARTERS:
+      duration = date_duration_t(date_duration_t::QUARTERS, quantity);
+      break;
+    case lexer_t::token_t::TOK_MONTHS:
+      duration = date_duration_t(date_duration_t::MONTHS, quantity);
+      break;
+    case lexer_t::token_t::TOK_WEEKS:
+      duration = date_duration_t(date_duration_t::WEEKS, quantity);
+      break;
+    case lexer_t::token_t::TOK_DAYS:
+      duration = date_duration_t(date_duration_t::DAYS, quantity);
+      break;
+    default:
+      tok.unexpected();
+      break;
+    }
+  } else {
+    switch (tok.kind) {
+    case lexer_t::token_t::TOK_YEAR:
+      duration = date_duration_t(date_duration_t::YEARS, 1);
+      break;
+    case lexer_t::token_t::TOK_QUARTER:
+      duration = date_duration_t(date_duration_t::QUARTERS, 1);
+      break;
+    case lexer_t::token_t::TOK_MONTH:
+      duration = date_duration_t(date_duration_t::MONTHS, 1);
+      break;
+    case lexer_t::token_t::TOK_WEEK:
+      duration = date_duration_t(date_duration_t::WEEKS, 1);
+      break;
+    case lexer_t::token_t::TOK_DAY:
+      duration = date_duration_t(date_duration_t::DAYS, 1);
+      break;
+    default:
+      tok.unexpected();
+      break;
+    }
+  }
+}
+
+void date_parser_t::handle_period_token(lexer_t::token_t::kind_t kind,
+                                        optional<date_duration_t>& duration)
+{
+  switch (kind) {
+  case lexer_t::token_t::TOK_YEARLY:
+    duration = date_duration_t(date_duration_t::YEARS, 1);
+    break;
+  case lexer_t::token_t::TOK_QUARTERLY:
+    duration = date_duration_t(date_duration_t::QUARTERS, 1);
+    break;
+  case lexer_t::token_t::TOK_BIMONTHLY:
+    duration = date_duration_t(date_duration_t::MONTHS, 2);
+    break;
+  case lexer_t::token_t::TOK_MONTHLY:
+    duration = date_duration_t(date_duration_t::MONTHS, 1);
+    break;
+  case lexer_t::token_t::TOK_BIWEEKLY:
+    duration = date_duration_t(date_duration_t::WEEKS, 2);
+    break;
+  case lexer_t::token_t::TOK_WEEKLY:
+    duration = date_duration_t(date_duration_t::WEEKS, 1);
+    break;
+  case lexer_t::token_t::TOK_DAILY:
+    duration = date_duration_t(date_duration_t::DAYS, 1);
+    break;
+  default:
+    break;
+  }
+}
+
+void date_parser_t::finalize_period(date_interval_t& period,
+                                    const optional<date_specifier_t>& since_specifier,
+                                    const optional<date_specifier_t>& until_specifier,
+                                    const optional<date_specifier_t>& inclusion_specifier,
+                                    bool end_inclusive)
+{
+  if (since_specifier || until_specifier) {
+    date_range_t range(since_specifier, until_specifier);
+    range.end_inclusive = end_inclusive;
+
+    period.range = date_specifier_or_range_t(range);
+    period.since_specified = static_cast<bool>(since_specifier);
+  }
+  else if (inclusion_specifier) {
+    period.range = date_specifier_or_range_t(*inclusion_specifier);
+  }
+  // else: it's something like "monthly", with no date reference
+}
+
 date_interval_t date_parser_t::parse()
 {
   optional<date_specifier_t> since_specifier;
@@ -801,303 +1178,67 @@ date_interval_t date_parser_t::parse()
   date_t          today = CURRENT_DATE();
   bool            end_inclusive = false;
 
+  // Main parsing loop - process tokens sequentially
   for (lexer_t::token_t tok = lexer.next_token();
        tok.kind != lexer_t::token_t::END_REACHED;
        tok = lexer.next_token()) {
+    
     switch (tok.kind) {
+    // Simple date/time specifiers
     case lexer_t::token_t::TOK_DATE:
-      if (! inclusion_specifier)
-        inclusion_specifier = date_specifier_t();
-      determine_when(tok, *inclusion_specifier);
-      break;
-
     case lexer_t::token_t::TOK_INT:
-      if (! inclusion_specifier)
-        inclusion_specifier = date_specifier_t();
-      determine_when(tok, *inclusion_specifier);
-      break;
-
     case lexer_t::token_t::TOK_A_MONTH:
-      if (! inclusion_specifier)
-        inclusion_specifier = date_specifier_t();
-      determine_when(tok, *inclusion_specifier);
-      break;
-
     case lexer_t::token_t::TOK_A_WDAY:
-      if (! inclusion_specifier)
-        inclusion_specifier = date_specifier_t();
-      determine_when(tok, *inclusion_specifier);
+      handle_date_token(tok, inclusion_specifier);
       break;
 
+    // Range operators
     case lexer_t::token_t::TOK_DASH:
-      if (inclusion_specifier) {
-        since_specifier     = inclusion_specifier;
-        until_specifier     = date_specifier_t();
-        inclusion_specifier = none;
-
-        tok = lexer.next_token();
-        determine_when(tok, *until_specifier);
-
-        // The dash operator is special: it has an _inclusive_ end.
-        end_inclusive = true;
-      } else {
-        tok.unexpected();
-      }
+      handle_dash_operator(tok, inclusion_specifier, since_specifier,
+                          until_specifier, end_inclusive);
       break;
 
     case lexer_t::token_t::TOK_SINCE:
-      if (since_specifier) {
-        tok.unexpected();
-      } else {
-        since_specifier = date_specifier_t();
-        tok = lexer.next_token();
-        determine_when(tok, *since_specifier);
-      }
+      handle_since_token(tok, since_specifier);
       break;
 
     case lexer_t::token_t::TOK_UNTIL:
-      if (until_specifier) {
-        tok.unexpected();
-      } else {
-        until_specifier = date_specifier_t();
-        tok = lexer.next_token();
-        determine_when(tok, *until_specifier);
-      }
+      handle_until_token(tok, until_specifier);
       break;
 
     case lexer_t::token_t::TOK_IN:
-      if (inclusion_specifier) {
-        tok.unexpected();
-      } else {
-        inclusion_specifier = date_specifier_t();
-        tok = lexer.next_token();
-        determine_when(tok, *inclusion_specifier);
-      }
+      handle_in_token(tok, inclusion_specifier);
       break;
 
+    // Relative time expressions (this/next/last)
     case lexer_t::token_t::TOK_THIS:
     case lexer_t::token_t::TOK_NEXT:
-    case lexer_t::token_t::TOK_LAST: {
-      int8_t adjust = 0;
-      if (tok.kind == lexer_t::token_t::TOK_NEXT)
-        adjust = 1;
-      else if (tok.kind == lexer_t::token_t::TOK_LAST)
-        adjust = -1;
-
-      tok = lexer.next_token();
-      switch (tok.kind) {
-      case lexer_t::token_t::TOK_INT: {
-        unsigned short amount = boost::get<unsigned short>(*tok.value);
-
-        date_t base(today);
-        date_t end(today);
-
-        if (! adjust)
-          adjust = 1;
-
-        tok = lexer.next_token();
-        switch (tok.kind) {
-        case lexer_t::token_t::TOK_YEARS:
-          base += gregorian::years(amount * adjust);
-          break;
-        case lexer_t::token_t::TOK_QUARTERS:
-          base += gregorian::months(amount * adjust * 3);
-          break;
-        case lexer_t::token_t::TOK_MONTHS:
-          base += gregorian::months(amount * adjust);
-          break;
-        case lexer_t::token_t::TOK_WEEKS:
-          base += gregorian::weeks(amount * adjust);
-          break;
-        case lexer_t::token_t::TOK_DAYS:
-          base += gregorian::days(amount * adjust);
-          break;
-        default:
-          tok.unexpected();
-          break;
-        }
-
-        if (adjust >= 0) {
-          date_t temp = base;
-          base = end;
-          end = temp;
-        }
-
-        since_specifier = date_specifier_t(base);
-        until_specifier = date_specifier_t(end);
-        break;
-      }
-
-      case lexer_t::token_t::TOK_A_MONTH: {
-        inclusion_specifier = date_specifier_t();
-        determine_when(tok, *inclusion_specifier);
-
-        date_t temp(today.year(), *inclusion_specifier->month, 1);
-        temp += gregorian::years(adjust);
-        inclusion_specifier =
-          date_specifier_t(static_cast<date_specifier_t::year_type>(temp.year()),
-                           temp.month());
-        break;
-      }
-
-      case lexer_t::token_t::TOK_A_WDAY: {
-        inclusion_specifier = date_specifier_t();
-        determine_when(tok, *inclusion_specifier);
-
-        date_t temp =
-          date_duration_t::find_nearest(today, date_duration_t::WEEKS);
-        while (temp.day_of_week() != inclusion_specifier->wday)
-          temp += gregorian::days(1);
-        temp += gregorian::days(7 * adjust);
-        inclusion_specifier = date_specifier_t(temp);
-        break;
-      }
-
-      case lexer_t::token_t::TOK_YEAR: {
-        date_t temp(today);
-        temp += gregorian::years(adjust);
-        inclusion_specifier =
-          date_specifier_t(static_cast<date_specifier_t::year_type>(temp.year()));
-        break;
-      }
-
-      case lexer_t::token_t::TOK_QUARTER: {
-        date_t base =
-          date_duration_t::find_nearest(today, date_duration_t::QUARTERS);
-        date_t temp;
-        if (adjust < 0) {
-          temp = base + gregorian::months(3 * adjust);
-        }
-        else if (adjust == 0) {
-          temp = base + gregorian::months(3);
-        }
-        else if (adjust > 0) {
-          base += gregorian::months(3 * adjust);
-          temp = base + gregorian::months(3 * adjust);
-        }
-        since_specifier = date_specifier_t(adjust < 0 ? temp : base);
-        until_specifier = date_specifier_t(adjust < 0 ? base : temp);
-        break;
-      }
-
-      case lexer_t::token_t::TOK_WEEK: {
-        date_t base =
-          date_duration_t::find_nearest(today, date_duration_t::WEEKS);
-        date_t temp;
-        if (adjust < 0) {
-          temp = base + gregorian::days(7 * adjust);
-        }
-        else if (adjust == 0) {
-          temp = base + gregorian::days(7);
-        }
-        else if (adjust > 0) {
-          base += gregorian::days(7 * adjust);
-          temp = base + gregorian::days(7 * adjust);
-        }
-        since_specifier = date_specifier_t(adjust < 0 ? temp : base);
-        until_specifier = date_specifier_t(adjust < 0 ? base : temp);
-        break;
-      }
-
-      case lexer_t::token_t::TOK_DAY: {
-        date_t temp(today);
-        temp += gregorian::days(adjust);
-        inclusion_specifier = date_specifier_t(temp);
-        break;
-      }
-
-      default:
-      case lexer_t::token_t::TOK_MONTH: {
-        date_t temp(today);
-        temp += gregorian::months(adjust);
-        inclusion_specifier =
-          date_specifier_t(static_cast<date_specifier_t::year_type>(temp.year()),
-                           temp.month());
-        break;
-      }
-      }
+    case lexer_t::token_t::TOK_LAST:
+      handle_relative_token(tok, since_specifier, until_specifier,
+                           inclusion_specifier, today);
       break;
-    }
 
+    // Simple period tokens (today/tomorrow/yesterday)
     case lexer_t::token_t::TOK_TODAY:
-      inclusion_specifier = date_specifier_t(today);
-      break;
     case lexer_t::token_t::TOK_TOMORROW:
-      inclusion_specifier = date_specifier_t(today + gregorian::days(1));
-      break;
     case lexer_t::token_t::TOK_YESTERDAY:
-      inclusion_specifier = date_specifier_t(today - gregorian::days(1));
+      handle_simple_period_token(tok.kind, inclusion_specifier, today);
       break;
 
+    // Recurring period specifications
     case lexer_t::token_t::TOK_EVERY:
-      tok = lexer.next_token();
-      if (tok.kind == lexer_t::token_t::TOK_INT) {
-        int quantity = boost::get<unsigned short>(*tok.value);
-        tok = lexer.next_token();
-        switch (tok.kind) {
-        case lexer_t::token_t::TOK_YEARS:
-          period.duration = date_duration_t(date_duration_t::YEARS, quantity);
-          break;
-        case lexer_t::token_t::TOK_QUARTERS:
-          period.duration = date_duration_t(date_duration_t::QUARTERS, quantity);
-          break;
-        case lexer_t::token_t::TOK_MONTHS:
-          period.duration = date_duration_t(date_duration_t::MONTHS, quantity);
-          break;
-        case lexer_t::token_t::TOK_WEEKS:
-          period.duration = date_duration_t(date_duration_t::WEEKS, quantity);
-          break;
-        case lexer_t::token_t::TOK_DAYS:
-          period.duration = date_duration_t(date_duration_t::DAYS, quantity);
-          break;
-        default:
-          tok.unexpected();
-          break;
-        }
-      } else {
-        switch (tok.kind) {
-        case lexer_t::token_t::TOK_YEAR:
-          period.duration = date_duration_t(date_duration_t::YEARS, 1);
-          break;
-        case lexer_t::token_t::TOK_QUARTER:
-          period.duration = date_duration_t(date_duration_t::QUARTERS, 1);
-          break;
-        case lexer_t::token_t::TOK_MONTH:
-          period.duration = date_duration_t(date_duration_t::MONTHS, 1);
-          break;
-        case lexer_t::token_t::TOK_WEEK:
-          period.duration = date_duration_t(date_duration_t::WEEKS, 1);
-          break;
-        case lexer_t::token_t::TOK_DAY:
-          period.duration = date_duration_t(date_duration_t::DAYS, 1);
-          break;
-        default:
-          tok.unexpected();
-          break;
-        }
-      }
+      handle_every_token(tok, period.duration);
       break;
 
+    // Period shortcuts
     case lexer_t::token_t::TOK_YEARLY:
-      period.duration = date_duration_t(date_duration_t::YEARS, 1);
-      break;
     case lexer_t::token_t::TOK_QUARTERLY:
-      period.duration = date_duration_t(date_duration_t::QUARTERS, 1);
-      break;
     case lexer_t::token_t::TOK_BIMONTHLY:
-      period.duration = date_duration_t(date_duration_t::MONTHS, 2);
-      break;
     case lexer_t::token_t::TOK_MONTHLY:
-      period.duration = date_duration_t(date_duration_t::MONTHS, 1);
-      break;
     case lexer_t::token_t::TOK_BIWEEKLY:
-      period.duration = date_duration_t(date_duration_t::WEEKS, 2);
-      break;
     case lexer_t::token_t::TOK_WEEKLY:
-      period.duration = date_duration_t(date_duration_t::WEEKS, 1);
-      break;
     case lexer_t::token_t::TOK_DAILY:
-      period.duration = date_duration_t(date_duration_t::DAYS, 1);
+      handle_period_token(tok.kind, period.duration);
       break;
 
     default:
@@ -1107,22 +1248,13 @@ date_interval_t date_parser_t::parse()
   }
 
 #if 0
-  if (! period.duration && inclusion_specifier)
+  if (!period.duration && inclusion_specifier)
     period.duration = inclusion_specifier->implied_duration();
 #endif
 
-  if (since_specifier || until_specifier) {
-    date_range_t range(since_specifier, until_specifier);
-    range.end_inclusive = end_inclusive;
-
-    period.range = date_specifier_or_range_t(range);
-  }
-  else if (inclusion_specifier) {
-    period.range = date_specifier_or_range_t(*inclusion_specifier);
-  }
-  else {
-    /* otherwise, it's something like "monthly", with no date reference */
-  }
+  // Finalize the period structure with collected specifiers
+  finalize_period(period, since_specifier, until_specifier,
+                  inclusion_specifier, end_inclusive);
 
   return period;
 }
@@ -1184,7 +1316,7 @@ date_t date_duration_t::find_nearest(const date_t& date, skip_quantum_t skip)
   return result;
 }
 
-void date_interval_t::stabilize(const optional<date_t>& date)
+void date_interval_t::stabilize(const optional<date_t>& date, bool align_intervals)
 {
 #if DEBUG_ON
   if (date)
@@ -1198,7 +1330,7 @@ void date_interval_t::stabilize(const optional<date_t>& date)
             "stabilize: aligning with a duration: " << *duration);
 
       // The interval object has not been seeded with a start date yet, so
-      // find the nearest period before on on date which fits, if possible.
+      // find the nearest period before or on date which fits, if possible.
       //
       // Find an efficient starting point for the upcoming while loop.  We
       // want a date early enough that the range will be correct, but late
@@ -1224,7 +1356,11 @@ void date_interval_t::stabilize(const optional<date_t>& date)
         // These start on most recent period start quantum before when
         DEBUG("times.interval",
             "stabilize: monthly, quarterly or yearly duration");
-        start = date_duration_t::find_nearest(when, duration->quantum);
+        if (align_intervals && since_specified) {
+          start = when;
+        } else {
+          start = date_duration_t::find_nearest(when, duration->quantum);
+        }
         break;
       case date_duration_t::WEEKS:
         // Weeks start on the beginning of week prior to 400 remainder period length
@@ -1232,9 +1368,13 @@ void date_interval_t::stabilize(const optional<date_t>& date)
         // implies period is never less than 400 days not too unreasonable
         DEBUG("times.interval", "stabilize: weekly duration");
         {
-          int period = duration->length * 7;
-          start = date_duration_t::find_nearest(
-              when - gregorian::days(period + 400 % period), duration->quantum);
+          if (align_intervals && since_specified) {
+            start = when;
+          } else {
+            int period = duration->length * 7;
+            start = date_duration_t::find_nearest(
+                when - gregorian::days(period + 400 % period), duration->quantum);
+          }
         }
         break;
       default:
@@ -1302,9 +1442,10 @@ void date_interval_t::stabilize(const optional<date_t>& date)
 }
 
 bool date_interval_t::find_period(const date_t& date,
+                                  const bool    align_intervals,
                                   const bool    allow_shift)
 {
-  stabilize(date);
+  stabilize(date, align_intervals);
 
   if (finish && date > *finish) {
     DEBUG("times.interval",
@@ -1471,7 +1612,7 @@ date_parser_t::lexer_t::token_t date_parser_t::lexer_t::next_token()
     return tok;
   }
 
-  while (begin != end && std::isspace(*begin))
+  while (begin != end && std::isspace(static_cast<unsigned char>(*begin)))
     begin++;
 
   if (begin == end)
@@ -1490,9 +1631,11 @@ date_parser_t::lexer_t::token_t date_parser_t::lexer_t::next_token()
   // date using the typical date formats.  This allows not only dates like
   // "2009/08/01", but also dates that fit the user's --input-date-format,
   // assuming their format fits in one argument and begins with a digit.
-  if (std::isdigit(*begin)) {
+  if (std::isdigit(static_cast<unsigned char>(*begin))) {
     string::const_iterator i = begin;
-    for (i = begin; i != end && ! std::isspace(*i); i++) {}
+    for (i = begin;
+         i != end && ! std::isspace(static_cast<unsigned char>(*i));
+         i++) {}
     assert(i != begin);
 
     string possible_date(start, i);
@@ -1517,18 +1660,20 @@ date_parser_t::lexer_t::token_t date_parser_t::lexer_t::next_token()
   start = begin;
 
   string term;
-  bool alnum = std::isalnum(*begin);
-  for (; (begin != end && ! std::isspace(*begin) &&
-          ((alnum && static_cast<bool>(std::isalnum(*begin))) ||
-           (! alnum && ! static_cast<bool>(std::isalnum(*begin))))); begin++)
+  bool alnum = std::isalnum(static_cast<unsigned char>(*begin));
+  for (; (begin != end && ! std::isspace(static_cast<unsigned char>(*begin)) &&
+          ((alnum && static_cast<bool>(std::isalnum(
+             static_cast<unsigned char>(*begin)))) ||
+           (! alnum && ! static_cast<bool>(std::isalnum(
+             static_cast<unsigned char>(*begin)))))); begin++)
     term.push_back(*begin);
 
   if (! term.empty()) {
-    if (std::isdigit(term[0])) {
+    if (std::isdigit(static_cast<unsigned char>(term[0]))) {
       return token_t(token_t::TOK_INT,
                      token_t::content_t(lexical_cast<unsigned short>(term)));
     }
-    else if (std::isalpha(term[0])) {
+    else if (std::isalpha(static_cast<unsigned char>(term[0]))) {
       to_lower(term);
 
       if (optional<date_time::months_of_year> month =
@@ -1561,7 +1706,7 @@ date_parser_t::lexer_t::token_t date_parser_t::lexer_t::next_token()
         return token_t(token_t::TOK_TODAY);
       else if (term == _("tomorrow"))
         return token_t(token_t::TOK_TOMORROW);
-      else if (term == _("yesterday"))
+      else if (term == _("yesterday") or term == _("yday"))
         return token_t(token_t::TOK_YESTERDAY);
       else if (term == _("year"))
         return token_t(token_t::TOK_YEAR);
@@ -1625,17 +1770,10 @@ void date_parser_t::lexer_t::token_t::unexpected()
 
 void date_parser_t::lexer_t::token_t::expected(char wanted, char c)
 {
-  if (c == '\0' || c == -1) {
-    if (wanted == '\0' || wanted == -1)
-      throw_(date_error, _("Unexpected end"));
-    else
-      throw_(date_error, _f("Missing '%1%'") % wanted);
-  } else {
-    if (wanted == '\0' || wanted == -1)
-      throw_(date_error, _f("Invalid char '%1%'") % c);
-    else
-      throw_(date_error, _f("Invalid char '%1%' (wanted '%2%')") % c % wanted);
-  }
+  if (wanted == '\0')
+    throw_(date_error, _f("Invalid char '%1%'") % c);
+  else
+    throw_(date_error, _f("Invalid char '%1%' (wanted '%2%')") % c % wanted);
 }
 
 namespace {
@@ -1704,11 +1842,13 @@ namespace {
 
 void set_datetime_format(const char * format)
 {
+  written_datetime_io->set_format(format);
   printed_datetime_io->set_format(format);
 }
 
 void set_date_format(const char * format)
 {
+  written_date_io->set_format(format);
   printed_date_io->set_format(format);
 }
 
@@ -1745,7 +1885,6 @@ void times_shutdown()
   if (is_initialized) {
     input_datetime_io.reset();
     timelog_datetime_io.reset();
-    input_date_io.reset();
     written_datetime_io.reset();
     written_date_io.reset();
     printed_datetime_io.reset();
