@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2014, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -41,8 +41,7 @@
  *
  * @brief datetime_t and date_t objects
  */
-#ifndef _TIMES_H
-#define _TIMES_H
+#pragma once
 
 #include "utils.h"
 
@@ -66,6 +65,7 @@ inline bool is_valid(const date_t& moment) {
 }
 
 extern optional<datetime_t> epoch;
+extern optional<int> year_directive_year;
 
 #ifdef BOOST_DATE_TIME_HAS_HIGH_PRECISION_CLOCK
 #define TRUE_CURRENT_TIME() (boost::posix_time::microsec_clock::local_time())
@@ -153,20 +153,6 @@ struct date_traits_t
             has_month == traits.has_month &&
             has_day     == traits.has_day);
   }
-
-#if HAVE_BOOST_SERIALIZATION
-private:
-  /** Serialization. */
-
-  friend class boost::serialization::access;
-
-  template<class Archive>
-  void serialize(Archive& ar, const unsigned int /* version */) {
-    ar & has_year;
-    ar & has_month;
-    ar & has_day;
-  }
-#endif // HAVE_BOOST_SERIALIZATION
 };
 
 struct date_duration_t
@@ -247,19 +233,6 @@ struct date_duration_t
   }
 
   static date_t find_nearest(const date_t& date, skip_quantum_t skip);
-
-#if HAVE_BOOST_SERIALIZATION
-private:
-  /** Serialization. */
-
-  friend class boost::serialization::access;
-
-  template<class Archive>
-  void serialize(Archive& ar, const unsigned int /* version */) {
-    ar & quantum;
-    ar & length;
-  }
-#endif // HAVE_BOOST_SERIALIZATION
 };
 
 class date_specifier_t
@@ -343,21 +316,6 @@ public:
 
     return out.str();
   }
-
-#if HAVE_BOOST_SERIALIZATION
-private:
-  /** Serialization. */
-
-  friend class boost::serialization::access;
-
-  template<class Archive>
-  void serialize(Archive& ar, const unsigned int /* version */) {
-    ar & year;
-    ar & month;
-    ar & day;
-    ar & wday;
-  }
-#endif // HAVE_BOOST_SERIALIZATION
 };
 
 class date_range_t
@@ -420,20 +378,6 @@ public:
 
     return out.str();
   }
-
-#if HAVE_BOOST_SERIALIZATION
-private:
-  /** Serialization. */
-
-  friend class boost::serialization::access;
-
-  template<class Archive>
-  void serialize(Archive& ar, const unsigned int /* version */) {
-    ar & range_begin;
-    ar & range_end;
-    ar & end_inclusive;
-  }
-#endif // HAVE_BOOST_SERIALIZATION
 };
 
 class date_specifier_or_range_t
@@ -490,18 +434,6 @@ public:
 
     return out.str();
   }
-
-#if HAVE_BOOST_SERIALIZATION
-private:
-  /** Serialization. */
-
-  friend class boost::serialization::access;
-
-  template<class Archive>
-  void serialize(Archive& ar, const unsigned int /* version */) {
-    ar & specifier_or_range;
-  }
-#endif // HAVE_BOOST_SERIALIZATION
 };
 
 class date_interval_t : public equality_comparable<date_interval_t>
@@ -520,6 +452,7 @@ public:
   optional<date_t>          next;
   optional<date_duration_t> duration;
   optional<date_t>          end_of_duration;
+  bool                      since_specified = false;
 
   explicit date_interval_t() : aligned(false) {
     TRACE_CTOR(date_interval_t, "");
@@ -535,7 +468,8 @@ public:
       aligned(other.aligned),
       next(other.next),
       duration(other.duration),
-      end_of_duration(other.end_of_duration) {
+      end_of_duration(other.end_of_duration),
+      since_specified(other.since_specified) {
     TRACE_CTOR(date_interval_t, "copy");
   }
   ~date_interval_t() throw() {
@@ -565,23 +499,20 @@ public:
   void   parse(const string& str);
 
   void   resolve_end();
-  void   stabilize(const optional<date_t>& date = none);
+  void   stabilize(const optional<date_t>& date = none, bool align_intervals = false);
 
   bool   is_valid() const {
-#if BOOST_VERSION >= 105600
-    return start != NULL;
-#else
-    return start;
-#endif
+    return static_cast<bool>(start);
   }
 
   /** Find the current or next period containing date.  Returns false if
       no such period can be found.  If allow_shift is true, the default,
       then the interval may be shifted in time to find the period. */
-  bool find_period(const date_t& date        = CURRENT_DATE(),
-                   const bool    allow_shift = true);
+  bool find_period(const date_t& date            = CURRENT_DATE(),
+                   const bool    align_intervals = false,
+                   const bool    allow_shift     = true);
   bool within_period(const date_t& date = CURRENT_DATE()) {
-    return find_period(date, false);
+    return find_period(date, false, false);
   }
 
   optional<date_t> inclusive_end() const {
@@ -594,24 +525,6 @@ public:
   date_interval_t& operator++();
 
   void dump(std::ostream& out);
-
-#if HAVE_BOOST_SERIALIZATION
-private:
-  /** Serialization. */
-
-  friend class boost::serialization::access;
-
-  template<class Archive>
-  void serialize(Archive& ar, const unsigned int /* version */) {
-    ar & range;
-    ar & start;
-    ar & finish;
-    ar & aligned;
-    ar & next;
-    ar & duration;
-    ar & end_of_duration;
-  }
-#endif // HAVE_BOOST_SERIALIZATION
 };
 
 void times_initialize();
@@ -622,5 +535,3 @@ void show_period_tokens(std::ostream& out, const string& arg);
 std::ostream& operator<<(std::ostream& out, const date_duration_t& duration);
 
 } // namespace ledger
-
-#endif // _TIMES_H
