@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2025, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -46,8 +46,7 @@
  * the number 10 to a value object, it's internal type will be
  * INTEGER.
  */
-#ifndef INCLUDED_VALUE_H
-#define INCLUDED_VALUE_H
+#pragma once
 
 #include "balance.h"            // includes amount.h
 #include "mask.h"
@@ -56,6 +55,7 @@ namespace ledger {
 
 DECLARE_EXCEPTION(value_error, std::runtime_error);
 
+class commodity_t;
 class scope_t;
 
 /**
@@ -77,10 +77,12 @@ class value_t
            equality_comparable<value_t, balance_t,
            additive<value_t, balance_t,
            multiplicative<value_t, balance_t,
+           totally_ordered<value_t, commodity_t,
+           addable<value_t, commodity_t,
            ordered_field_operators<value_t, amount_t,
            ordered_field_operators<value_t, double,
            ordered_field_operators<value_t, unsigned long,
-           ordered_field_operators<value_t, long> > > > > > > >
+           ordered_field_operators<value_t, long> > > > > > > > > >
 {
 public:
   /**
@@ -105,6 +107,7 @@ public:
     INTEGER,                    // a signed integer value
     AMOUNT,                     // a ledger::amount_t
     BALANCE,                    // a ledger::balance_t
+    COMMODITY,                  // a ledger::commodity_t
     STRING,                     // a string object
     MASK,                       // a regular expression mask
     SEQUENCE,                   // a vector of value_t objects
@@ -126,17 +129,18 @@ public:
      * The `type' member holds the value_t::type_t value representing
      * the type of the object stored.
      */
-    variant<bool,               // BOOLEAN
-            datetime_t,         // DATETIME
-            date_t,             // DATE
-            long,               // INTEGER
-            amount_t,           // AMOUNT
-            balance_t *,        // BALANCE
-            string,             // STRING
-            mask_t,             // MASK
-            sequence_t *,       // SEQUENCE
-            scope_t *,          // SCOPE
-            boost::any          // ANY
+    variant<bool,                // BOOLEAN
+            datetime_t,          // DATETIME
+            date_t,              // DATE
+            long,                // INTEGER
+            amount_t,            // AMOUNT
+            balance_t *,         // BALANCE
+            commodity_t const *, // COMMODITY
+            string,              // STRING
+            mask_t,              // MASK
+            sequence_t *,        // SEQUENCE
+            scope_t *,           // SCOPE
+            boost::any           // ANY
             > data;
 
     type_t type;
@@ -309,6 +313,10 @@ public:
     set_balance(val);
     TRACE_CTOR(value_t, "const balance_t&");
   }
+  value_t(const commodity_t& val) {
+    set_commodity(val);
+    TRACE_CTOR(value_t, "const commodity_t&");
+  }
   value_t(const mask_t& val) {
     set_mask(val);
     TRACE_CTOR(value_t, "const mask_t&");
@@ -651,6 +659,15 @@ public:
     storage->data = new balance_t(val);
   }
 
+  bool is_commodity() const {
+    return is_type(COMMODITY);
+  }
+  const commodity_t& as_commodity() const {
+    VERIFY(is_commodity());
+    return *boost::get<commodity_t const *>(storage->data);
+  }
+  void set_commodity(const commodity_t& val);
+
   bool is_string() const {
     return is_type(STRING);
   }
@@ -783,6 +800,7 @@ public:
   date_t      to_date() const;
   amount_t    to_amount() const;
   balance_t   to_balance() const;
+  const commodity_t& to_commodity() const;
   string      to_string() const;
   mask_t      to_mask() const;
   sequence_t  to_sequence() const;
@@ -997,5 +1015,3 @@ bool sort_value_is_less_than(const std::list<sort_value_t>& left_values,
 void put_value(property_tree::ptree& pt, const value_t& value);
 
 } // namespace ledger
-
-#endif // INCLUDED_VALUE_H
