@@ -1,10 +1,9 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
+#!/usr/bin/env python3
 
+import argparse
 import sys
 import re
 import os
-import argparse
 
 from os.path import *
 from subprocess import Popen, PIPE
@@ -52,24 +51,9 @@ class CheckBaselineTests (CheckOptions):
     return errors
 
 if __name__ == "__main__":
-  def getargs():
-    parser = argparse.ArgumentParser(prog='CheckBaselineTests',
-            description='Check that ledger options are tested')
-    parser.add_argument('-l', '--ledger',
-        dest='ledger',
-        type=str,
-        action='store',
-        required=True,
-        help='the path to the ledger executable to test with')
-    parser.add_argument('-s', '--source',
-        dest='source',
-        type=str,
-        action='store',
-        required=True,
-        help='the path to the top level ledger source directory')
-    return parser.parse_args()
-
-  args = getargs()
+  args = argparse.ArgumentParser(prog='CheckBaselineTests',
+                                 description='Check that ledger options are tested',
+                                 parents=[CheckOptions.parser()]).parse_args()
   script = CheckBaselineTests(args)
   status = script.main()
   sys.exit(status)
